package mpi.eudico.client.annotator.layout;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JSplitPane;
import javax.swing.SwingConstants;
import javax.swing.border.SoftBevelBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import mpi.eudico.client.annotator.Constants;
import mpi.eudico.client.annotator.ElanFrame2;
import mpi.eudico.client.annotator.ElanLayoutManager;
import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.ElanMediaPlayerController;
import mpi.eudico.client.annotator.Preferences;
import mpi.eudico.client.annotator.SelectionPanel;
import mpi.eudico.client.annotator.ViewerManager2;
import mpi.eudico.client.annotator.commands.Command;
import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.client.annotator.gui.ResizeComponent;
import mpi.eudico.client.annotator.player.EmptyMediaPlayer;
import mpi.eudico.client.annotator.transcriptionMode.PossibleTypesExtractor;
import mpi.eudico.client.annotator.transcriptionMode.TranscriptionModeSettingsDlg;
import mpi.eudico.client.annotator.transcriptionMode.TranscriptionViewer;
import mpi.eudico.client.annotator.util.FrameConstants;
import mpi.eudico.client.annotator.viewer.SignalViewer;
import mpi.eudico.server.corpora.clom.Transcription;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;


/**
 * Creates transcription mode layout.
 *
 * @author Aarthy Somasundaram
 */
public class TranscriptionManager implements ModeLayoutManager, ActionListener, ChangeListener {
						
	private ElanLayoutManager layoutManager;
    private Container container;
    private ViewerManager2 viewerManager; 
    private Transcription transcription;
    private SignalViewer signalViewer;
	private ResizeComponent horMediaResizer;
	private ElanMediaPlayerController mediaPlayerController;
	private TranscriptionViewer transViewer;
	
	// volume rate panel
    private JPanel volRatePanel;
    private JLabel volNameLabel;    
    private JLabel volValueLabel;
    private JLabel rateNameLabel;
    private JLabel rateValueLabel;
    private JSlider volSlider;
    private JSlider rateSlider;
    
    //settings panel
    private JScrollPane settingsScrollPane;
    private JCheckBox autoPlayBackCB;
    private JCheckBox showTierNamesCB;
    private JCheckBox showColorOnlyOnNoColumnCB;
    private JCheckBox moveViaColumnCB;
    private JCheckBox showActiveCellInCenterCB;
    private JButton changeTypesButton; 
    
    private JSplitPane tableSplitPane;
    
    private JButton butPlay;   
    private JButton butAlignVideoToRight;
    private JCheckBox loopModeCB;
    
    private Icon playIcon;
    private Icon pauseIcon;
    private Icon alignLeftIcon;
	private Icon alignRightIcon;
	
	private final String toopTipAlignLeft = ElanLocale.getString("TranscriptionManager.AlignVideoLeft");
	private final String toopTipAlignRight = ElanLocale.getString("TranscriptionManager.AlignVideoRight");
	
	private boolean videoInRight = false;
	private boolean initialized = false;
	private int numVisualPlayers;
	
	private static int DEF_SIGNAL_HEIGHT = 200;	
	private final static Dimension BUTTON_SIZE = new Dimension(30, 20);	
	public final static String COMBOBOX_DEFAULT_STRING = ElanLocale.getString("TranscriptionManager.ComboBoxDefaultString");
		
	private float zoomLevelSignalViewer = -1F;
	private float zoomLevelTimeLineViewer = -1F;
	
	/**
	 * Creates an instance of the TranscriptionManager
	 * 
	 * @param viewerManager
	 * @param elanLayoutManager
	 */
    public TranscriptionManager(ViewerManager2 viewerManager, ElanLayoutManager elanLayoutManager) {
        this.viewerManager = viewerManager;
        this.layoutManager = elanLayoutManager;
        transcription = viewerManager.getTranscription();        
        if(viewerManager.getMasterMediaPlayer() != null){
        	viewerManager.getMasterMediaPlayer().setMediaTime(0);
        }
        
        container = layoutManager.getContainer();    
        
        // divider component
        horMediaResizer = new ResizeComponent(layoutManager, SwingConstants.HORIZONTAL);
        horMediaResizer.setBorder(new SoftBevelBorder(SoftBevelBorder.RAISED));
        horMediaResizer.setPreferredSize(new Dimension(8,container.getHeight()));
        Component n = horMediaResizer.getComponent(0);
        horMediaResizer.remove(n);
        horMediaResizer.setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.anchor = gbc.PAGE_END;
        gbc.weighty = 1.0; 
        horMediaResizer.add(n, gbc);
        
    	container.add(horMediaResizer);
    }
	
    /**
     * Adds the object to the layout
     */
	public void add(Object object) {
		if (object instanceof SignalViewer) {
	    	setSignalViewer((SignalViewer) object);
	    } else if(object instanceof ElanMediaPlayerController){
	    	setMediaPlayerController ((ElanMediaPlayerController) object);
	    } 
	}	
	
	/**
	 * DOCUMENT ME!
	 *
	 * @param signalViewer
	 */
	private void setSignalViewer(SignalViewer signalViewer){
		this.signalViewer  = signalViewer;	
		signalViewer.setEnabled(false);
		signalViewer.setMediaTime(0);
		signalViewer.setSelection(0,0);
        container.add(signalViewer);
        signalViewer.setBorder(new SoftBevelBorder(SoftBevelBorder.LOWERED));
        if(initialized){ 
        	doLayout();
        	transViewer.updateSignalViewer(signalViewer);        
        }
	}	
	
	/**
     * DOCUMENT ME!
     *
     * @param mediaPlayerController
     */
    private void setMediaPlayerController(
        ElanMediaPlayerController mediaPlayerController) {
        this.mediaPlayerController = mediaPlayerController;        
 
        container.add(mediaPlayerController.getSelectionPanel());
        ((SelectionPanel)mediaPlayerController.getSelectionPanel()).setNameLabel(
        		ElanLocale.getString("TranscriptionManager.TimeInterval"));
        
        JComponent timePanel = mediaPlayerController.getTimePanel();
        MouseListener listeners[] = timePanel.getMouseListeners();
        if(listeners !=null){
        	for(int i=0; i< listeners.length; i++){
        		timePanel.removeMouseListener(listeners[i]);
        	}
        }
        container.add(mediaPlayerController.getTimePanel());   
        
        Component[] s = mediaPlayerController.getSelectionButtonsPanel().getComponents();
        for(int i=0; i< s.length; i++){
        	s[i].setEnabled(false);
        	if( ((JButton)s[i]).getAction() == ELANCommandFactory.getCommandAction(
                    transcription,   ELANCommandFactory.SELECTION_BOUNDARY)){
        		((JButton)s[i]).setVisible(false);
        	}
        }        
        container.add(mediaPlayerController.getSelectionButtonsPanel());                
    }
    
    /**
     * Removes the object from the layout
     */
	public void remove(Object object) {
		if (object instanceof SignalViewer && signalViewer != null) {
			container.remove(signalViewer);
			signalViewer = null;
		}
	}

	/**
	 * makes the layout for this mode
	 */
	public void doLayout() {				
		int containerWidth 		= container.getWidth();
	    int containerHeight 	= container.getHeight();
	    int containerMargin 	= 3;
	    int componentMargin 	= 5;
	    
	    int mediaAreaWidth 		= layoutManager.getMediaAreaWidth();
	    int visibleMediaX 		= containerMargin;
	    int visibleMediaY 		= containerMargin;
	    int visibleMediaWidth 	= mediaAreaWidth;
	    int visibleMediaHeight 	= 0;
	    int firstMediaWidth	    = mediaAreaWidth;
	    
	    PlayerLayoutModel[] visualPlayers = layoutManager.getAttachedVisualPlayers();
		numVisualPlayers = visualPlayers.length;
		
		// if there is no media
		if(numVisualPlayers == 0){
			if(videoInRight){
				visibleMediaX = containerWidth - (visibleMediaWidth+ containerMargin);
			} 
		} 
		
		if (numVisualPlayers >= 1) {
			// layout the first video
			Component firstVisualComp = visualPlayers[0].visualComponent;
			float aspectRatio = visualPlayers[0].player.getAspectRatio();					
			if(numVisualPlayers >= 2){
			firstMediaWidth = mediaAreaWidth / 2;
			}			
			int firstMediaHeight = (int) (firstMediaWidth / aspectRatio);						
			visibleMediaWidth = firstMediaWidth;	
			visibleMediaHeight = firstMediaHeight;
			if(videoInRight){	
				visibleMediaX = (containerWidth - (visibleMediaWidth+containerMargin));
			}			
			firstVisualComp.setBounds(visibleMediaX, visibleMediaY, firstMediaWidth,
						firstMediaHeight);
			if(numVisualPlayers > 1){
				visibleMediaWidth += componentMargin;	
			}
		}
		
		if (numVisualPlayers == 2) {			
			Component secondVisualComp = visualPlayers[1].visualComponent;
			float aspectRatio = visualPlayers[1].player.getAspectRatio();			
			int secondMediaWidth = (int) (visibleMediaHeight * aspectRatio);
			int secondMediaHeight = visibleMediaHeight;
			if(videoInRight){				
				visibleMediaX = (containerWidth - (visibleMediaWidth + secondMediaWidth + containerMargin));
				visualPlayers[0].visualComponent.setBounds(visibleMediaX, visibleMediaY, firstMediaWidth,
						visibleMediaHeight);	
			}			
			secondVisualComp.setBounds(visibleMediaX + visibleMediaWidth, visibleMediaY, secondMediaWidth, secondMediaHeight);
			visibleMediaWidth += secondMediaWidth;	
		}
		else if (numVisualPlayers == 3) {			
			Component secondVisualComp = visualPlayers[1].visualComponent;
			float secondAR = visualPlayers[1].player.getAspectRatio();
			Component thirdVisualComp = visualPlayers[2].visualComponent;
			float thirdAR = visualPlayers[2].player.getAspectRatio();
			int heightPerPlayer = (visibleMediaHeight - componentMargin) / 2;
			int secondWidth = (int)(secondAR * heightPerPlayer);
			int thirdWidth = (int) (thirdAR * heightPerPlayer);
			int widthPerPlayer = Math.max(secondWidth, thirdWidth);
			if(videoInRight){
				visibleMediaX = (containerWidth - (visibleMediaWidth + widthPerPlayer + containerMargin));
				visualPlayers[0].visualComponent.setBounds(visibleMediaX, visibleMediaY, firstMediaWidth,
						visibleMediaHeight);			
			}
			secondVisualComp.setBounds(visibleMediaX + visibleMediaWidth + (widthPerPlayer - secondWidth) / 2, 
					visibleMediaY, secondWidth, heightPerPlayer);
			thirdVisualComp.setBounds(visibleMediaX + visibleMediaWidth + (widthPerPlayer - thirdWidth) / 2, 
					visibleMediaY + heightPerPlayer + componentMargin, 	thirdWidth, heightPerPlayer);			
			visibleMediaWidth += widthPerPlayer;
		}
		else if (numVisualPlayers >= 4) {
			Component secondVisualComp = visualPlayers[1].visualComponent;
			float secondAR = visualPlayers[1].player.getAspectRatio();
			Component thirdVisualComp = visualPlayers[2].visualComponent;
			float thirdAR = visualPlayers[2].player.getAspectRatio();
			Component fourthVisualComp = visualPlayers[3].visualComponent;
			float fourthAR = visualPlayers[3].player.getAspectRatio();
			int heightPerPlayer = (visibleMediaHeight - 2 * componentMargin) / 3;
			int secondWidth = (int)(secondAR * heightPerPlayer);
			int thirdWidth = (int) (thirdAR * heightPerPlayer);
			int fourthWidth = (int) (fourthAR * heightPerPlayer);
			int widthPerPlayer = Math.max(secondWidth, thirdWidth);
			widthPerPlayer = Math.max(widthPerPlayer, fourthWidth);
		
			if(videoInRight){
				visibleMediaX = (containerWidth - (visibleMediaWidth + widthPerPlayer + containerMargin));
				visualPlayers[0].visualComponent.setBounds(visibleMediaX, visibleMediaY, firstMediaWidth,
						visibleMediaHeight);
			}
		
			secondVisualComp.setBounds(visibleMediaX + visibleMediaWidth + 
					(widthPerPlayer - secondWidth) / 2, visibleMediaY, 
					secondWidth, heightPerPlayer);
			thirdVisualComp.setBounds(visibleMediaX + visibleMediaWidth + 
					(widthPerPlayer - thirdWidth) / 2, 
					visibleMediaY + heightPerPlayer + componentMargin, 
					thirdWidth, heightPerPlayer);
			fourthVisualComp.setBounds(visibleMediaX + visibleMediaWidth + 
					(widthPerPlayer - fourthWidth) / 2, 
					visibleMediaY + 2 * heightPerPlayer + 2 * componentMargin, 
					fourthWidth, heightPerPlayer);
			visibleMediaWidth += widthPerPlayer + componentMargin;
		}
		
		// layout for the time panel
		int timePanelX = visibleMediaX;
		int timePanelY = visibleMediaY + visibleMediaHeight + 2;
		int timePanelWidth = 0;
		int timePanelHeight = 0;        
		if (mediaPlayerController != null) {
			timePanelWidth = mediaPlayerController.getTimePanel().getPreferredSize().width;
			timePanelHeight = mediaPlayerController.getTimePanel().getPreferredSize().height;
			if(numVisualPlayers > 0){	
				timePanelX = (visibleMediaX + (firstMediaWidth / 2)) - (timePanelWidth / 2);
			} 					
			mediaPlayerController.getTimePanel().setBounds(timePanelX, timePanelY, timePanelWidth, timePanelHeight);
		}
        
		//layout for the video Alignment button
		int butAlignVideoX = visibleMediaX;
		int butAlignVideoY = timePanelY + timePanelHeight + 4;
		int butAlignVideoWidth = butAlignVideoToRight.getPreferredSize().width;
		int butAlignVideoHeight = butAlignVideoToRight.getPreferredSize().height;       
		butAlignVideoToRight.setBounds(butAlignVideoX, butAlignVideoY, butAlignVideoWidth, butAlignVideoHeight);	

		// layout for the buttons
		int playButtonX = butAlignVideoX + butAlignVideoWidth;
		int playButtonY = butAlignVideoY;
		int playButtonWidth = butPlay.getPreferredSize().width;
		int playButtonHeight = butPlay.getPreferredSize().height;
        if(numVisualPlayers > 0 ){		
		// playButtonWidth*4 === width of all 4 buttons			
			playButtonX = (visibleMediaX + (firstMediaWidth / 2)) -
								((playButtonWidth*4) / 2);	
		}
		if (playButtonX < (butAlignVideoX + butAlignVideoWidth)) {
			playButtonX = butAlignVideoX + butAlignVideoWidth;
		}
		butPlay.setBounds(playButtonX, playButtonY, playButtonWidth, playButtonHeight);   
        
		//layout for the selection buttons panel
		int selButtonPanelX = playButtonX+playButtonWidth;;
		int selButtonPanelY = playButtonY;
		int selButtonPanelWidth = 0;
		int selButtonPanelHeight = 0;
        if (mediaPlayerController != null) {
			selButtonPanelWidth = mediaPlayerController.getSelectionButtonsPanel().getPreferredSize().width;
			selButtonPanelHeight = mediaPlayerController.getSelectionButtonsPanel().getPreferredSize().height;			
			mediaPlayerController.getSelectionButtonsPanel().setBounds(selButtonPanelX, selButtonPanelY, 
					selButtonPanelWidth, selButtonPanelHeight);
		}	
			
		//layout for the loop mode check box
		int  loopModeCBX = selButtonPanelX + selButtonPanelWidth;
		int  loopModeCBY = selButtonPanelY;
		int  loopModeCBWidth = loopModeCB.getPreferredSize().width;
		int  loopModeCBHeight = loopModeCB.getPreferredSize().height;
        if(numVisualPlayers > 0 || signalViewer !=null) {
			loopModeCBX = (visibleMediaX + visibleMediaWidth) -loopModeCBWidth ;        	
		}        
		if(loopModeCBX < (selButtonPanelX + selButtonPanelHeight)){
			loopModeCBX = visibleMediaX;
			loopModeCBY = selButtonPanelY + selButtonPanelHeight + 5;
		}
		loopModeCB.setBounds(loopModeCBX, loopModeCBY, loopModeCBWidth, loopModeCBHeight);	
	          
		//layout for the time interval panel
		int selPanelX = visibleMediaX;
		int selPanelY = loopModeCBY + loopModeCBHeight + 10;// was 25
		int selPanelWidth = 0;
		int selPanelHeight = 0;
        if (mediaPlayerController!= null) {
			selPanelWidth = mediaPlayerController.getSelectionPanel().getPreferredSize().width;
			selPanelHeight = mediaPlayerController.getSelectionPanel().getPreferredSize().height;        	
			if(signalViewer != null){
    			selPanelX = visibleMediaX + (visibleMediaWidth / 2) - (selPanelWidth / 2);
			}
			mediaPlayerController.getSelectionPanel().setBounds(selPanelX, selPanelY, selPanelWidth, selPanelHeight);
		}
        
		// layout signal Viewer
		int signalX = visibleMediaX;
		int signalY = selPanelY + selPanelHeight + 10;
		int signalWidth = visibleMediaWidth;
        int signalHeight = 0;
        
        if (signalViewer != null) {   
        	signalHeight = DEF_SIGNAL_HEIGHT/2;
        	signalViewer.setBounds(signalX, signalY, signalWidth, signalHeight);
        } 
        
        //	layout for volume, rate panel  
        int volX = visibleMediaX;
		int volY = signalY + signalHeight + 10;
		int volWidth = visibleMediaWidth;
		int volHeight = DEF_SIGNAL_HEIGHT/2;		
		getVolumeRatePanel().setBounds(volX, volY, volWidth, volHeight);
		
		int settingsPanelX = visibleMediaX;
		int settingsPanelY = volY + volHeight + 10;
		int settingsPanelWidth = visibleMediaWidth;
		int settingsPanelHeight = getSettingsScrollPane().getPreferredSize().height + getSettingsScrollPane().getHorizontalScrollBar().getPreferredSize().height;
		
		int availableSpace = containerHeight - containerMargin - settingsPanelY;
		
		if (settingsPanelHeight > availableSpace) {
			settingsPanelHeight = availableSpace;
		} 		
		
		getSettingsScrollPane().setBounds(settingsPanelX, settingsPanelY, settingsPanelWidth, settingsPanelHeight);
       
		
		// resize divider       
        int divX = visibleMediaX + visibleMediaWidth + containerMargin; // always keep visible
        int divY = containerHeight - horMediaResizer.getPreferredSize().height; 
        if(videoInRight){
        	divX = visibleMediaX - containerMargin- horMediaResizer.getPreferredSize().width;
        }
        horMediaResizer.setBounds(divX, divY, horMediaResizer.getPreferredSize().width,  
        		horMediaResizer.getPreferredSize().height);     
		
        if(viewerManager.getMasterMediaPlayer() instanceof EmptyMediaPlayer && signalViewer == null){
        	if (mediaPlayerController.getTimePanel() != null) {				
        		mediaPlayerController.getTimePanel().setBounds(0, 0, 0, 0);
        	}	
    	
        	butPlay.setBounds(0, 0, 0, 0);
        	if (mediaPlayerController.getSelectionButtonsPanel() != null) {
        		mediaPlayerController.getSelectionButtonsPanel().setBounds(0, 0, 0, 0);
        	}
        	loopModeCB.setBounds(0, 0, 0, 0);
        	if (mediaPlayerController.getSelectionPanel()!= null) {
        		mediaPlayerController.getSelectionPanel().setBounds(0, 0, 0, 0);
        	}
        	
        	getVolumeRatePanel().setBounds(0,0,0,0);
           
    		settingsPanelY = butAlignVideoY + butAlignVideoHeight + 4;
    		getSettingsScrollPane().setBounds(settingsPanelX, settingsPanelY, settingsPanelWidth, settingsPanelHeight);
        } 
		
        // layout for the viewer
	    int transPanelX = divX + horMediaResizer.getPreferredSize().width+ containerMargin;
        int transPanelY = containerMargin;
        int transPanelWidth = containerWidth - transPanelX - containerMargin;
        int transPanelHeight = containerHeight - transPanelY - containerMargin;  
        
        if(videoInRight){
        	transPanelX = containerMargin;
        	transPanelWidth = containerWidth - visibleMediaWidth - (3*containerMargin)  - horMediaResizer.getPreferredSize().width;
    	}        
        
        transViewer.setBounds(transPanelX, transPanelY, transPanelWidth, transPanelHeight);
        transViewer.setScrollerSize(transPanelWidth,transPanelHeight);        
		
        container.validate();
        //container.repaint();
	}
	
	public void detach(Object object) {
		transViewer.focusTable();
	}

	public void attach(Object object) {
		transViewer.focusTable();
	}	

	public void updateLocale() {
		if (settingsScrollPane != null) {
			changeTypesButton.setText(ElanLocale.getString("TranscriptionManager.ChangeSettings"));
			autoPlayBackCB.setText(ElanLocale.getString("TranscriptionManager.AutoPlayBack"));
			showActiveCellInCenterCB.setText(ElanLocale.getString("TranscriptionManager.ShowActiveCellInCenter"));
			showTierNamesCB.setText(ElanLocale.getString("TranscriptionManager.ShowTierNames"));
			showColorOnlyOnNoColumnCB.setText(ElanLocale.getString("TranscriptionManager.ShowColorOnlyOnNoColumn"));
			moveViaColumnCB.setText(ElanLocale.getString("TranscriptionManager.MoveViaColumn"));
		}
		if (volRatePanel != null) {
			volNameLabel.setText(ElanLocale.getString("MediaPlayerControlPanel.ElanSlider.Volume"));
			rateNameLabel.setText(ElanLocale.getString("MediaPlayerControlPanel.ElanSlider.Rate"));
		}
		if (mediaPlayerController != null) {
	        ((SelectionPanel)mediaPlayerController.getSelectionPanel()).setNameLabel(
	        		ElanLocale.getString("TranscriptionManager.TimeInterval"));
		}
	}

	/**
	 * Removes all the components in this layout
	 */
	public void clearLayout() {		
		
		if(mediaPlayerController.getLoopMode()){
			mediaPlayerController.setLoopMode(false);
		}
		viewerManager.getMasterMediaPlayer().stop();
		transViewer.isClosing();
		container.remove(horMediaResizer);		
		container.remove(transViewer);		
		container.remove(butPlay);
		container.remove(butAlignVideoToRight);	
		container.remove(loopModeCB);
		container.remove(getVolumeRatePanel());
		container.remove(getSettingsScrollPane());
		container.remove(mediaPlayerController.getTimePanel());
		container.remove(mediaPlayerController.getSelectionPanel());
		((SelectionPanel)mediaPlayerController.getSelectionPanel()).updateLocale();
		
		Component[] s = mediaPlayerController.getSelectionButtonsPanel().getComponents();
        for(int i=0; i< s.length; i++){
        	s[i].setEnabled(true);
        	if( ((JButton)s[i]).getAction() == ELANCommandFactory.getCommandAction(
                    transcription,   ELANCommandFactory.SELECTION_BOUNDARY)){
        		((JButton)s[i]).setVisible(true);        		
        	}
        }
        
        container.remove(mediaPlayerController.getSelectionButtonsPanel());
	
		viewerManager.destroyTimePanel();
		viewerManager.destroyTranscriptionViewer();
		transViewer= null;
		
		if(signalViewer != null){	
			signalViewer.setRecalculateInterval(true);
			signalViewer.setBorder(null);
			signalViewer.updateInterval(0, viewerManager.getMasterMediaPlayer().getMediaDuration());
			signalViewer.setEnabled(true);
			signalViewer.preferencesChanged();			
			container.remove(signalViewer);
		}		
		container.repaint();
	}
	
	public void cleanUpOnClose() {	
	}

	/**
	 * Initialize the required components for this mode
	 */
	public void initComponents() {
		
		Object zoom = Preferences.get("SignalViewer.ZoomLevel",transcription);
		if (zoom instanceof Float) {
			zoomLevelSignalViewer = (Float) zoom;
		}else {
			zoomLevelSignalViewer = -1F;
		}
		
		zoom = Preferences.get("TimeLineViewer.ZoomLevel",transcription);
		if (zoom instanceof Float) {
			zoomLevelTimeLineViewer = (Float) zoom;
		}else {
			zoomLevelTimeLineViewer = -1F;
		}

		
		transViewer = viewerManager.createTranscriptionViewer();	
		transViewer.intializeViewer(this);	
		container.add(transViewer);		
		
		container.add(getVolumeRatePanel());
		
		container.add(getSettingsScrollPane());
	    
	    playIcon = new ImageIcon(this.getClass().getResource("/mpi/eudico/client/annotator/resources/PlayButton.gif"));
        pauseIcon = new ImageIcon(this.getClass().getResource("/mpi/eudico/client/annotator/resources/PauseButton.gif"));       	    
	    butPlay = new JButton(playIcon);
	    butPlay.setPreferredSize(BUTTON_SIZE);
	    butPlay.setEnabled(false);
	    butPlay.addActionListener(this);
	    container.add(butPlay); 
	    
	    alignLeftIcon = new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/AlignLeft16.gif"));
        alignRightIcon = new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/AlignRight16.gif"));
        
        butAlignVideoToRight = new JButton();
	    butAlignVideoToRight.setIcon(alignRightIcon);
	    butAlignVideoToRight.setPreferredSize(BUTTON_SIZE);	   
	    butAlignVideoToRight.addActionListener(this);	 
	    butAlignVideoToRight.setToolTipText(toopTipAlignRight);
	    container.add(butAlignVideoToRight); 
	    
	    loopModeCB = new JCheckBox(ElanLocale.getString(ELANCommandFactory.LOOP_MODE));
	    loopModeCB.setFont(Constants.SMALLFONT);
	    loopModeCB.addActionListener(this);	
	    container.add(loopModeCB);
	    
	    layoutManager.add(viewerManager.getMediaPlayerController());  
	    
	    // adding a signal viewer
	    createAndAddViewer(ELANCommandFactory.SIGNAL_VIEWER);	    
	    Object val = Preferences.get("TranscriptionManager.PlaceVideoInRight", transcription);
	    if(val != null){
	    	if((Boolean)val){
	    		alignVideo();
	    	}
	    }
	    
	    doLayout();
	    
		loadPreferences();
		initialized = true;	
	}
	
	public ElanLayoutManager getElanLayoutManager(){
		return layoutManager;
	}
	
	 private JSplitPane getTableSplitPane() {
	        if (tableSplitPane == null) {
	        	tableSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
	        	tableSplitPane.setOneTouchExpandable(true);

	            // HS 24 nov set the divider location when a top component is added
	        	tableSplitPane.setDividerLocation(0);
	        	tableSplitPane.setContinuousLayout(true);
	            container.add(tableSplitPane);
	        }

	        return tableSplitPane;
	    }
	
	private JScrollPane getSettingsScrollPane() {
		if (settingsScrollPane != null) {
			return settingsScrollPane;
		}
		
		JPanel settingsPanel = new JPanel(new GridBagLayout());
		
		changeTypesButton = new JButton(ElanLocale.getString("TranscriptionManager.ChangeSettings"));
		changeTypesButton.addActionListener(this);
        
        autoPlayBackCB = new JCheckBox(ElanLocale.getString("TranscriptionManager.AutoPlayBack"));
        autoPlayBackCB.setSelected(true);
        autoPlayBackCB.addActionListener(this);       
	    
	    showActiveCellInCenterCB = new JCheckBox(ElanLocale.getString("TranscriptionManager.ShowActiveCellInCenter"));
	    showActiveCellInCenterCB.setSelected(false);
	    showActiveCellInCenterCB.addActionListener(this);	  
        
	    showTierNamesCB = new JCheckBox(ElanLocale.getString("TranscriptionManager.ShowTierNames"));
	    showTierNamesCB.setSelected(true);	    
	    showTierNamesCB.addActionListener(this);	
	    
	    showColorOnlyOnNoColumnCB = new JCheckBox(ElanLocale.getString("TranscriptionManager.ShowColorOnlyOnNoColumn"));
	    showColorOnlyOnNoColumnCB.setSelected(false);	 
	    showColorOnlyOnNoColumnCB.setEnabled(false);
	    showColorOnlyOnNoColumnCB.addActionListener(this);
	    
	    moveViaColumnCB = new JCheckBox(ElanLocale.getString("TranscriptionManager.MoveViaColumn"));		
		moveViaColumnCB.addActionListener(this);	    
		
		Insets insets = new Insets(1, 1, 1, 1);  	
    	
    	GridBagConstraints gbc = new GridBagConstraints();    	
    	gbc.insets = insets;
    	gbc.anchor = GridBagConstraints.NORTHWEST;
    	gbc.fill = GridBagConstraints.NONE;    	
    	settingsPanel.add(autoPlayBackCB, gbc);
    	
    	gbc.gridy = 1;       	
    	settingsPanel.add(showTierNamesCB, gbc);
    	
    	gbc.gridy = 2; 
    	gbc.insets = new Insets(1, 22, 1, 1);
    	settingsPanel.add(showColorOnlyOnNoColumnCB, gbc);
    	
    	gbc.gridy = 3;    
    	gbc.insets = insets;
    	settingsPanel.add(moveViaColumnCB, gbc);
    	
    	gbc.gridy = 4;    	
    	settingsPanel.add(showActiveCellInCenterCB, gbc);
    	
    	gbc.gridy = 5;  
    	//gbc.weighty = 1.0;	
    	gbc.weightx= 1.0;	
    	gbc.anchor = GridBagConstraints.WEST;
    	settingsPanel.add(changeTypesButton, gbc);

    	settingsPanel.setBorder(null);  
		
		settingsScrollPane = new JScrollPane(settingsPanel);
		settingsScrollPane.setBorder(new TitledBorder(ElanLocale.getString("TranscriptionManager.ChangeSettingsDlg.Title")));	
		settingsScrollPane.setBackground(settingsPanel.getBackground());		
		
		
		return settingsScrollPane;
	}
	
	/**
	 * Returns the volume and the rate sliders panel
	 * 
	 * @return
	 */
    private JPanel getVolumeRatePanel() {
    	if (volRatePanel != null) {
    		return volRatePanel;
    	}
    	volRatePanel = new JPanel(new GridBagLayout());
    	volNameLabel = new JLabel(ElanLocale.getString("MediaPlayerControlPanel.ElanSlider.Volume"));
    	int volume = (int)(viewerManager.getMasterMediaPlayer().getVolume() * 100);
    	volValueLabel = new JLabel(String.valueOf(volume));
    	volValueLabel.setFont(volValueLabel.getFont().deriveFont(Font.PLAIN, 10));
    	volSlider = new JSlider(0, 100, volume);
    	volSlider.addChangeListener(this);
    	
    	rateNameLabel = new JLabel(ElanLocale.getString("MediaPlayerControlPanel.ElanSlider.Rate"));
    	int rate = (int)(viewerManager.getMasterMediaPlayer().getRate() * 100);
    	rateValueLabel = new JLabel(String.valueOf(rate));
    	rateValueLabel.setFont(rateValueLabel.getFont().deriveFont(Font.PLAIN, 10));
    	rateSlider = new JSlider(0, 200, rate);
    	rateSlider.addChangeListener(this);
    	
    	GridBagConstraints gbc = new GridBagConstraints();
    	Insets insets = new Insets(1, 1, 1, 1);
    	gbc.insets = insets;
    	gbc.anchor = GridBagConstraints.SOUTHWEST;
    	volRatePanel.add(volNameLabel, gbc);
    	
    	gbc.gridy = 2;
    	gbc.gridwidth = 1;
    	gbc.fill = GridBagConstraints.NONE;    	
    	gbc.insets = new Insets(1, 1, 1, 1);
    	volRatePanel.add(rateNameLabel, gbc);
    	
    	gbc.gridx = 1;
    	gbc.gridy = 0;
    	gbc.insets = new Insets(1, 6, 1, 1);
    	gbc.weighty = 0.5;	    
    	volRatePanel.add(volValueLabel, gbc);
    	
    	gbc.gridy = 2;    	
    	volRatePanel.add(rateValueLabel, gbc);
    	
    	gbc.gridx = 0;
    	gbc.gridy = 1;
    	gbc.gridwidth = 2;
    	gbc.insets = insets;
    	gbc.fill = GridBagConstraints.HORIZONTAL;
    	gbc.weightx = 1.0;
    	volRatePanel.add(volSlider, gbc);  
    	
    	gbc.gridy = 3;
    	volRatePanel.add(rateSlider, gbc);
    	
    	volRatePanel.setBorder(new TitledBorder(""));    	
    	return volRatePanel;
    }
    
    /**
     * Loads the user preferences
     */
    private void loadPreferences(){		
		transViewer.loadPreferences();
		
		Object val = Preferences.get("TranscriptionManager.MoveViaColumn", null);
		if(val != null){
			moveViaColumnCB.setSelected((Boolean) val);	
			transViewer.moveViaColumn((Boolean) val);
		}	
		
		val = Preferences.get("TranscriptionManager.AutoPlayBack", null);
		if(val != null){
			autoPlayBackCB.setSelected((Boolean) val);
			transViewer.setAutoPlayBack((Boolean) val);
		}
		
		val = Preferences.get("TranscriptionManager.LoopMode", null);
		if(val != null){
			loopModeCB.setSelected((Boolean) val);
			if(loopModeCB.isSelected()){
				changeLoopMode();
			}
		}
		
		val = Preferences.get("TranscriptionManager.ScrollToCenter", null);
		if(val != null){
			showActiveCellInCenterCB.setSelected((Boolean) val);
			transViewer.scrollActiveCellInCenter((Boolean) val);
		}
		
		val = Preferences.get("TranscriptionManager.ShowTierNames", null);
		if(val != null){
			showTierNamesCB.setSelected((Boolean) val);
			transViewer.showTierNames((Boolean)val);
			if(showTierNamesCB.isSelected()){
				showColorOnlyOnNoColumnCB.setEnabled(false);
			} else {
				showColorOnlyOnNoColumnCB.setEnabled(true);
			}
		}
		
		val = Preferences.get("TranscriptionManager.ShowColorOnlyOnNoColumn", null);
		if(val != null){
			showColorOnlyOnNoColumnCB.setSelected((Boolean) val);
			transViewer.showColorOnlyOnNoColumn((Boolean)val);
		}				
		 
		val = Preferences.get("TranscriptionTable.ColumnTypes", transcription);
		List<String> types = null;
		if(val instanceof List){
			types = (List<String>)val;
		} else {
			List<String> colTypes = new ArrayList<String>();
			val = Preferences.get("TranscriptionTable.Column1Type", transcription);
			if(val != null){
				colTypes.add(val.toString());
				val = Preferences.get("TranscriptionTable.Column2Type", transcription);
				if(val != null){
					colTypes.add(val.toString());
					val = Preferences.get("TranscriptionTable.Column3Type", transcription);
					if(val != null){
						colTypes.add(val.toString());	
					}
				}
			}
			
			if(colTypes.size() >0){
				types = colTypes; 
			}
		}		
		
		// validate the type list
		List<String> validTypes = new ArrayList<String>();
		List<Integer> deletedTypeIndexList = new ArrayList<Integer>();
		if(types != null && types.size() > 0){
			PossibleTypesExtractor computeTypes = new PossibleTypesExtractor(viewerManager.getTranscription());
	    	
	    	List<String> possibleTypes = computeTypes.getPossibleTypesForColumn(1, validTypes);	    	
	    	if(possibleTypes.contains(types.get(0))){
				validTypes.add(types.get(0));
				for(int i = 1; i < types.size(); i++){
					possibleTypes = computeTypes.getPossibleTypesForColumn(validTypes.size()+1, validTypes);
					if(possibleTypes.contains(types.get(i))){
						validTypes.add(types.get(i));
					} else {
						deletedTypeIndexList.add(i);
					}
				}
			}
		}
		
		val = Preferences.get("TranscriptionTable.FontSize", null);	
		
		if(validTypes == null && validTypes.size() <= 0){
			TranscriptionModeSettingsDlg settingsDialog = new TranscriptionModeSettingsDlg(layoutManager, null,null, null, (Integer)val);
			settingsDialog.setVisible(true);						
			applyChanges(settingsDialog.getTierMap(), settingsDialog.getHiddenTiersList(), settingsDialog.getColumnTypeList(), settingsDialog.getFontSize(), settingsDialog.isValueChanged());
		} else {
			
			HashMap<TierImpl, List<TierImpl>> tierMap = null;
			List<String> hiddenTiersList = null;
			HashMap<String, List<String>> map = (HashMap<String, List<String>>) Preferences.get("TranscriptionTable.TierMap", viewerManager.getTranscription());
			if(map != null){			
				tierMap = getValidatedTierMap(map, deletedTypeIndexList);	
			}
			
			if(tierMap != null){
				List<String> valList = (List<String>) Preferences.get("TranscriptionTable.HiddenTiers", viewerManager.getTranscription());
				if(valList != null){
					hiddenTiersList = getValidatedTierList(valList);			
				}
				
				valList = (List<String>) Preferences.get("TranscriptionTable.NonEditableTiers", viewerManager.getTranscription());
				if(valList != null){			
					transViewer.setNoneditableTier(getValidatedTierList(valList));	
				}
			}
			
			
			
			applyChanges(tierMap, hiddenTiersList, validTypes, (Integer)val, true);
			transViewer.updateTable();
		}
    }
    
    private HashMap<TierImpl, List<TierImpl>> getValidatedTierMap(HashMap<String, List<String>> map, List<Integer> deletedTypeIndexList){
    	HashMap<TierImpl, List<TierImpl>> validMap = new HashMap<TierImpl, List<TierImpl>>();
		List<String> tierNamesList = null;
		List<TierImpl> tierList = null;
		String keyObj;
		Iterator<String> keyIt = map.keySet().iterator();
		
		List<String> validTierNameList;
    	
		TierImpl referenceTier = null;
    	while (keyIt.hasNext()) {
    		keyObj = keyIt.next();	
    		referenceTier = null;
    		if(map.get(keyObj) instanceof List){
    			tierNamesList =  map.get(keyObj);
    			referenceTier = (TierImpl)viewerManager.getTranscription().getTierWithId(keyObj);
    			if(referenceTier != null){
    				tierList = new ArrayList<TierImpl>();
    				
    				for(int i=0; i< tierNamesList.size(); i++){
    					if(deletedTypeIndexList.contains(i)){
    						continue;
    					}    					
    					String tierName = tierNamesList.get(i);
    					if(tierName.equals("No tier")){
    						tierList.add(null);    					
    					} else if(tierName.equals(keyObj)){
    						tierList.add(referenceTier);    					
    					} else {
    						tierList.add((TierImpl)viewerManager.getTranscription().getTierWithId(tierName));    	
    					}
    				}    
    				validMap.put(referenceTier, tierList);
    			}
    		}
    	}
    	
    	if(validMap.keySet().size() <= 0){
    		return null;
    	}
    	return validMap;
    		
    }
    
    private List<String> getValidatedTierList(List<String> tierNames){
		List<String> tierList = new ArrayList<String>();
		String tierName;
		for(int i = 0; i < tierNames.size(); i++){
			tierName = tierNames.get(i);
			if(viewerManager.getTranscription().getTierWithId(tierName) != null){
				tierList.add(tierNames.get(i));
			}
		}		
		return tierList;
	}
    
    /**
     * Returns whether the all the components fro this mode
     * is initialized
     * 
     * @return initialized, if true initialized else
     * 						false - not initialized
     */
	public boolean isInitialized(){
		return initialized;
	}
	
	/**
	 * Switches to annotation mode for editing the
	 * active annotation
	 */
	public void editInAnnotationMode(){
		((ElanFrame2)layoutManager.getElanFrame()).setMenuSelected(
				ElanLocale.getString(ELANCommandFactory.ANNOTATION_MODE), FrameConstants.OPTION);
		layoutManager.changeMode(ElanLayoutManager.NORMAL_MODE);
	}
	
	/**
	 *  Set the preferences
	 */
	public void setPreference(String key, Object value, Object document) {
		if (document instanceof Transcription) {
			Preferences.set(key, value, (Transcription)document, false, false);
		} else {
			Preferences.set(key, value, null, false, false);
		}
	}
	
	/**
	 * Applies the new type/size changes to the transcription 
	 * table
	 * @param hashMap 
	 * @param b 
	 * @param hashMap 
	 * 
	 * @param type1, type for column 1
	 * @param type2, type for column 2
	 * @param type3, type for column 3
	 * @param size, font size for the table
	 */
	private void applyChanges(HashMap<TierImpl, List<TierImpl>> hashMap, List<String> hiddenTiers, List<String> types, Integer size, boolean valueChanged){				
		if(types == null ){
			transViewer.focusTable();
			return;
		}
		
		if(valueChanged){
			transViewer.setColumnTypeList(types);			
			transViewer.setTierMap(hashMap);
			transViewer.setHiddenTiersList(hiddenTiers);
		}				
				
		if(size != null){
			if(size != transViewer.getFontSize()){				
				transViewer.setFontSize(size);
			}
			setPreference("TranscriptionTable.FontSize", transViewer.getFontSize(), null);	
		}
		
		if(valueChanged){			
			transViewer.loadTable();
			transViewer.checkForMerge();
		} 	
		
		transViewer.reValidateTable();
		transViewer.focusTable();
	}
	
	public void preferencesChanged() {
	}
	
	/**
	 * Enables or disables the menu's related to the current layout
	 * 
	 * @param enabled
	 */
	public void enableOrDisableMenus(boolean enabled){
		((ElanFrame2)layoutManager.getElanFrame()).enableOrDisableMenus(null, FrameConstants.ANNOTATION, enabled);
		
		List<String> searchMenu = new ArrayList<String>();
		searchMenu.add(ElanLocale.getString(ELANCommandFactory.SEARCH_DLG));
		((ElanFrame2)layoutManager.getElanFrame()).enableOrDisableMenus(searchMenu, FrameConstants.SEARCH, enabled);

	}
	
    /**
     * Sets the icon of the play button 
     *     
     * @param play
     */
    public void setPlayPauseButton(boolean play) {
        if (play) {
            butPlay.setIcon(playIcon);
        } else {
        	 butPlay.setIcon(pauseIcon);
        }
       transViewer.focusTable();
    }
    
    /**
     * Enables/Disables the Play button
     * 
     * @param b
     */
    public void enablePlayButton(boolean b) {
		butPlay.setEnabled(b);		
		Component c[] = mediaPlayerController.getSelectionButtonsPanel().getComponents();
		if(c!= null){
			for(int i=0; i< c.length; i++){
				c[i].setEnabled(b);
			}
		}
	} 
    
    /**
     * Switch on/off the loop mode
     */
    public void toggleLoopMode() {
		loopModeCB.setSelected(!loopModeCB.isSelected());		
		changeLoopMode();
	}
	
    /**
     * Changes the loop mode
     */
    private void changeLoopMode(){
    	Object[] args = new Object[2];
    	args[0] = mediaPlayerController;
    	args[1] = viewerManager.getMasterMediaPlayer();			
    	Command c = ELANCommandFactory.createCommand(transcription, ELANCommandFactory.LOOP_MODE);
    	c.execute(null, args);			
    	transViewer.focusTable();
    	setPreference("TranscriptionManager.LoopMode", loopModeCB.isSelected(), null);	
    }
    
	public void stateChanged(ChangeEvent e) {
		if (e.getSource() == volSlider) {
			int volume = volSlider.getValue();
			volValueLabel.setText(String.valueOf(volume));			
			viewerManager.getMasterMediaPlayer().setVolume(((float) volume / 100));			
		} 
		else if (e.getSource() == rateSlider) {
			int rate = rateSlider.getValue();
			rateValueLabel.setText(String.valueOf(rate));
			viewerManager.getMasterMediaPlayer().setRate(((float) rate / 100));			
		}
		transViewer.focusTable();
	}   
	
	private void alignVideo(){
		videoInRight = !videoInRight;
		horMediaResizer.changeBehaviourToDecrease(videoInRight) ;
		if(videoInRight){
			butAlignVideoToRight.setToolTipText(toopTipAlignLeft);
			butAlignVideoToRight.setIcon(alignLeftIcon);				
		}else{
			butAlignVideoToRight.setToolTipText(toopTipAlignRight);
			butAlignVideoToRight.setIcon(alignRightIcon);	
		}
		doLayout();
		setPreference("TranscriptionManager.PlaceVideoInRight", videoInRight, transcription);
		transViewer.focusTable();
	}
	
	public void actionPerformed(ActionEvent e) {		
		if(e.getSource()== butPlay){
			transViewer.playMedia(false);
		} 
		else if(e.getSource() == loopModeCB){
			changeLoopMode();
		} 
		else if (e.getSource() == butAlignVideoToRight){
			alignVideo();
		} 
		else if(e.getSource() == changeTypesButton){
			viewerManager.getMasterMediaPlayer().stop();
			TranscriptionModeSettingsDlg settingsDialog = new TranscriptionModeSettingsDlg(layoutManager, transViewer.getColumnTypes(), transViewer.getTierMap(), transViewer.getHiddenTiers(), transViewer.getFontSize());
			settingsDialog.setVisible(true);
			applyChanges(settingsDialog.getTierMap() , settingsDialog.getHiddenTiersList(), settingsDialog.getColumnTypeList(), settingsDialog.getFontSize(), settingsDialog.isValueChanged());
		} 
		else {			
			if(viewerManager.getMasterMediaPlayer().isPlaying()){
				viewerManager.getMasterMediaPlayer().start();
			}		
			
			if(e.getSource() == showTierNamesCB){
				transViewer.showTierNames(showTierNamesCB.isSelected());
				setPreference("TranscriptionManager.ShowTierNames", showTierNamesCB.isSelected(), null);
				if(showTierNamesCB.isSelected()){
					showColorOnlyOnNoColumnCB.setEnabled(false);
				} else {
					showColorOnlyOnNoColumnCB.setEnabled(true);
				}
			}
			else if(e.getSource() == showColorOnlyOnNoColumnCB){
				transViewer.showColorOnlyOnNoColumn(showColorOnlyOnNoColumnCB.isSelected());
				setPreference("TranscriptionManager.ShowColorOnlyOnNoColumn", showColorOnlyOnNoColumnCB.isSelected(), null);
			}
			else if(e.getSource() == moveViaColumnCB){
				transViewer.moveViaColumn(moveViaColumnCB.isSelected());		
				setPreference("TranscriptionManager.MoveViaColumn", moveViaColumnCB.isSelected(), null);
			} 
			else if(e.getSource() == autoPlayBackCB){	
				transViewer.setAutoPlayBack(autoPlayBackCB.isSelected());		
				setPreference("TranscriptionManager.AutoPlayBack", autoPlayBackCB.isSelected(), null);    
			} 
			else if(e.getSource() == showActiveCellInCenterCB){
				transViewer.scrollActiveCellInCenter(showActiveCellInCenterCB.isSelected());	
				setPreference("TranscriptionManager.ScrollToCenter", showActiveCellInCenterCB.isSelected(), null);
			}		
			transViewer.focusTable();
		}
	}
	
	public void shortcutsChanged() {
		transViewer.shortcutsChanged();
		
	}

	public void createAndAddViewer(String viewerName) {
		if(viewerName == null){
			return;
		}
		if (viewerName.equals(ELANCommandFactory.SIGNAL_VIEWER)) {
			layoutManager.add(viewerManager.createSignalViewer());
		}
	}

	public boolean destroyAndRemoveViewer(String viewerName) {
		if(viewerName == null){
			return false;
		}
		if (viewerName.equals(ELANCommandFactory.SIGNAL_VIEWER) && signalViewer != null) {
			container.remove(signalViewer);
			signalViewer = null;
			viewerManager.destroySignalViewer();	
			return true;
		}		
		return false;		
	}
	
	public void isClosing() {
		transViewer.isClosing();		
		transViewer.setActiveAnnotation();
		if(signalViewer != null){	
			signalViewer.setRecalculateInterval(true);
		}
		
		if(zoomLevelSignalViewer > -1F){
			setPreference("SignalViewer.ZoomLevel", zoomLevelSignalViewer, transcription);
		} else {
			setPreference("SignalViewer.ZoomLevel", new Float(100.0f), transcription);
		}
		
		if(zoomLevelTimeLineViewer > -1F){
			setPreference("TimeLineViewer.ZoomLevel", zoomLevelTimeLineViewer, transcription);
		} else {
			setPreference("TimeLineViewer.ZoomLevel", new Float(100.0f), transcription);
		}	
	}
}
