package mpi.eudico.client.annotator.prefs.gui;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.Preferences;

import mpi.eudico.client.annotator.gui.FileChooser;
import mpi.eudico.client.annotator.prefs.PreferenceEditor;

import mpi.eudico.client.annotator.util.FileUtility;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.File;

import java.util.HashMap;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Panel showing option to change the media navigation setting: <br>
 * - frame forward/backward jumps to the begin of next/previous frame (default
 * it jumps with the amount of ms of the duration of a single frame)  - a
 * default location (directory) to search for media files (after the
 * "traditional" location)
 *
 * @author Han Sloetjes
 * @version 2.0, Dec 2008
 */
public class MediaNavPanel extends JPanel implements PreferenceEditor, ChangeListener,
    ActionListener {
    private boolean origFrameStepToFrameBegin = false;
    private String curMediaLocation = "-";
    private boolean origVideoSameSize = false;
    private boolean videoInCentre = false;
    private boolean origAltMediaLocSetsDirty = true;
    private String origTimeFormat = MS;    
    private JCheckBox frameStepCB;
    private JCheckBox videosSameSizeCB;
    private JCheckBox videosInCentreCB;
    private JLabel setDirLabel;
    private JLabel curDirLabel;
    private JButton defaultDirButton;
    private JButton resetDirButton;
    private JCheckBox changedMediaLocCB;
    private JLabel timeFormatLabel;
    private JComboBox timeFormatComboBox;
   
    private static String HH_MM_SS_MS = ElanLocale.getString("TimeCodeFormat.Hours");
    private static String SS_MS = ElanLocale.getString("TimeCodeFormat.Seconds");
    private static String MS = ElanLocale.getString("TimeCodeFormat.MilliSec");   
    private static String NTSC = ElanLocale.getString("TimeCodeFormat.TimeCode.SMPTE.NTSC");   
    private static String PAL = ElanLocale.getString("TimeCodeFormat.TimeCode.SMPTE.PAL");   
    
    /**
     * Creates a new MediaNavPanel instance
     */
    public MediaNavPanel() {
        super();
        readPrefs();
        initComponents();
    }

    private void readPrefs() {
        Object val = Preferences.get("MediaNavigation.FrameStepToFrameBegin",
                null);

        if (val instanceof Boolean) {
            origFrameStepToFrameBegin = ((Boolean) val).booleanValue();
        }

        val = Preferences.get("DefaultMediaLocation", null);

        if (val instanceof String) {
            curMediaLocation = (String) val;
        }

        val = Preferences.get("Media.VideosSameSize", null);

        if (val instanceof Boolean) {
            origVideoSameSize = ((Boolean) val).booleanValue();
        }
        
        val = Preferences.get("Media.VideosCentre", null);

        if (val instanceof Boolean) {
            videoInCentre = ((Boolean) val).booleanValue();
        }
        
        val = Preferences.get("MediaLocation.AltLocationSetsChanged", null);
        
        if (val instanceof Boolean) {
        	origAltMediaLocSetsDirty = ((Boolean) val).booleanValue();
        }
        
        val = Preferences.get("CurrentTime.Copy.TimeFormat", null);
        
        if (val instanceof String) {
        	origTimeFormat = ((String) val).toString();
        }
    }

    private void initComponents() {
        setLayout(new GridBagLayout());

        Insets insets = new Insets(2, 0, 2, 0);
        frameStepCB = new JCheckBox(ElanLocale.getString(
                    "PreferencesDialog.MediaNav.FrameBegin"),
                origFrameStepToFrameBegin);
        frameStepCB.setFont(frameStepCB.getFont().deriveFont(Font.PLAIN));

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 1.0;
        gbc.insets = insets;
        gbc.gridwidth = 3;
        add(new JLabel(ElanLocale.getString(
                    "PreferencesDialog.Category.MediaNav")), gbc);

        gbc.gridy = 1;
        add(frameStepCB, gbc);

        gbc.gridy = 2;
        gbc.insets = new Insets(12, 0, 2, 0);
        add(new JLabel(ElanLocale.getString(
                    "PreferencesDialog.Media.VideoDisplay")), gbc);
        videosSameSizeCB = new JCheckBox(ElanLocale.getString(
                    "PreferencesDialog.Media.VideoSize"), origVideoSameSize);
        gbc.gridy = 3;
        gbc.insets = insets;
        videosSameSizeCB.setFont(videosSameSizeCB.getFont()
                                                 .deriveFont(Font.PLAIN));
        add(videosSameSizeCB, gbc);
        
        videosInCentreCB = new JCheckBox(ElanLocale.getString(
        	"PreferencesDialog.Media.VideoCentre"), videoInCentre );
        //videosInCentreCB.addChangeListener(this);
        videosInCentreCB.addActionListener(this);
        gbc.gridy = 4;
        gbc.insets = insets;
        videosInCentreCB.setFont(videosInCentreCB.getFont().deriveFont(Font.PLAIN));
        add(videosInCentreCB, gbc);
        
        

        gbc.gridy = 5;
        gbc.insets = new Insets(12, 0, 2, 0);
        add(new JLabel(ElanLocale.getString("PreferencesDialog.Media.Location")),
            gbc);
        setDirLabel = new JLabel(ElanLocale.getString(
                    "PreferencesDialog.Media.DefaultLoc"));
        setDirLabel.setFont(setDirLabel.getFont().deriveFont(Font.PLAIN));
        gbc.gridy = 6;
        gbc.gridwidth = 1;
        gbc.weightx = 0.5;
        gbc.insets = insets;
        add(setDirLabel, gbc);
        curDirLabel = new JLabel(curMediaLocation);
        curDirLabel.setFont(curDirLabel.getFont().deriveFont(Font.PLAIN));
        gbc.gridy = 7;
        add(curDirLabel, gbc);

        defaultDirButton = new JButton(ElanLocale.getString("Button.Browse"));
        gbc.gridx = 1;
        gbc.gridy = 6;
        gbc.fill = GridBagConstraints.NONE;
        gbc.weightx = 0.0;
        gbc.gridheight = 2;
        gbc.anchor = GridBagConstraints.NORTHEAST;
        add(defaultDirButton, gbc);
        defaultDirButton.addActionListener(this);

        resetDirButton = new JButton();

        ImageIcon resetIcon = null;

        // add reset icon
        try {
            resetIcon = new ImageIcon(this.getClass()
                                          .getResource("/mpi/eudico/client/annotator/resources/Remove.gif"));
            resetDirButton.setIcon(resetIcon);
        } catch (Exception ex) {
            resetDirButton.setText("X");
        }

        resetDirButton.setToolTipText(ElanLocale.getString(
                "PreferencesDialog.Reset"));
        resetDirButton.setPreferredSize(new Dimension(
                resetDirButton.getPreferredSize().width,
                defaultDirButton.getPreferredSize().height));
        gbc.gridx = 2;
        gbc.gridy = 6;
        gbc.fill = GridBagConstraints.NONE;
        gbc.weightx = 0.0;
        gbc.gridheight = 2;
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.insets = new Insets(2, 10, 2, 0);
        add(resetDirButton, gbc);
        resetDirButton.addActionListener(this);
        
        changedMediaLocCB = new JCheckBox(ElanLocale.getString("PreferencesDialog.Media.SaveAltLocation"));
        changedMediaLocCB.setFont(changedMediaLocCB.getFont().deriveFont(Font.PLAIN));
        changedMediaLocCB.setSelected(origAltMediaLocSetsDirty);

        gbc.gridx = 0;
        gbc.gridy = 8;
        gbc.insets = insets;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 1.0;
        gbc.gridwidth = 3;
        add(changedMediaLocCB, gbc); 
        
        timeFormatLabel = new JLabel(ElanLocale.getString("PreferencesDialog.Media.TimeFormat"));
        gbc.gridy = 10;
        gbc.gridwidth = 1;
        gbc.insets = new Insets(12, 0, 2, 0);
        add(timeFormatLabel, gbc);
        
        timeFormatComboBox = new JComboBox();
        timeFormatComboBox.addItem(HH_MM_SS_MS);        
        timeFormatComboBox.addItem(SS_MS);
        timeFormatComboBox.addItem(MS);    
        timeFormatComboBox.addItem(NTSC);    
        timeFormatComboBox.addItem(PAL);    
        timeFormatComboBox.setSelectedItem(origTimeFormat);
        gbc.gridx=1;
        gbc.gridwidth = 2;
        add(timeFormatComboBox, gbc);
        
        gbc.gridx = 0;
        gbc.gridy = 11;
        gbc.gridwidth = 3;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weighty = 1.0;
        gbc.weightx = 1.0;
        add(new JPanel(), gbc); // filler
    }

    /**
     * Returns the changed pref.
     *
     * @return a map with the changed pref, or null
     */
    public Map getChangedPreferences() {
        if (isChanged()) {
            Map chMap = new HashMap(4);

            if (frameStepCB.isSelected() != origFrameStepToFrameBegin) {
                chMap.put("MediaNavigation.FrameStepToFrameBegin",
                    new Boolean(frameStepCB.isSelected()));
            }

            if (videosSameSizeCB.isSelected() != origVideoSameSize) {
                chMap.put("Media.VideosSameSize",
                    new Boolean(videosSameSizeCB.isSelected()));
            }
            
            if (videosInCentreCB.isSelected() != videoInCentre) {
                chMap.put("Media.VideosCentre",
                    new Boolean(videosInCentreCB.isSelected()));
            }

            if ((curDirLabel.getText() != null) &&
                    !curDirLabel.getText().equals("-")) {
                chMap.put("DefaultMediaLocation", curDirLabel.getText());
            } else {
                chMap.put("DefaultMediaLocation", null);
            }
            
            if (changedMediaLocCB.isSelected() != origAltMediaLocSetsDirty) {
            	chMap.put("MediaLocation.AltLocationSetsChanged", 
            			new Boolean(changedMediaLocCB.isSelected()));
            }   
            
            if(!timeFormatComboBox.getSelectedItem().toString().equals(origTimeFormat)){
            	chMap.put("CurrentTime.Copy.TimeFormat", 
            			new String(timeFormatComboBox.getSelectedItem().toString()));
            }
            return chMap;
        }

        return null;
    }

    /**
     * Returns whether anything has changed.
     *
     * @return whether anything has changed
     */
    public boolean isChanged() {
        return ((frameStepCB.isSelected() != origFrameStepToFrameBegin) ||
        !curMediaLocation.equals(curDirLabel.getText()) ||
        (videosSameSizeCB.isSelected() != origVideoSameSize) ||
        (videosInCentreCB.isSelected() != videoInCentre) ||
        (changedMediaLocCB.isSelected() != origAltMediaLocSetsDirty) || 
        (!origTimeFormat.equals(timeFormatComboBox.getSelectedItem().toString())));
    }

    /**
     * Action event handling
     *
     * @param e the event
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == defaultDirButton) {
            // show a folder file chooser, set the current def. location
            FileChooser chooser = new FileChooser(this);
            if (curMediaLocation.length() > 1) {
                File dir = new File(FileUtility.urlToAbsPath(curMediaLocation));

                if (dir.exists() && dir.isDirectory()) {
                	 chooser.setCurrentDirectory(dir.getAbsolutePath());
                }
            }
           chooser.createAndShowFileDialog(ElanLocale.getString("PreferencesDialog.Media.DefaultLoc"), FileChooser.OPEN_DIALOG, ElanLocale.getString("Button.Select"), 
        		   null, null, true, null, FileChooser.DIRECTORIES_ONLY, null);
            //chooser.setMultiSelectionEnabled(false);
           
           File selFile = chooser.getSelectedFile();
           if (selFile != null) {
               curDirLabel.setText(selFile.getAbsolutePath());
               curDirLabel.setText(FileUtility.pathToURLString(
                       selFile.getAbsolutePath()));
           } 
            
        } else if (e.getSource() == resetDirButton) {
            curDirLabel.setText("-");
        } else if (e.getSource() == videosInCentreCB) {
        	Preferences.set("Media.VideosCentre.Temporary", videosInCentreCB.isSelected(), null);
        }
    }


	public void stateChanged(ChangeEvent e) {// on Windows this is triggered by mouse hover etc. 
		if(e.getSource() == videosInCentreCB ){
			Preferences.set("Media.VideosCentre.Temporary", videosInCentreCB.isSelected(), null);	
		}
		
	}
	
}
